//
//  KAddon.m
//  KEdit
//
//  Created by Karino, Masatsugu on Tue May 20 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KAddon.h"
#import "KStringUtils.h"

@implementation KAddon

// initializers.
- (id)init
{
	[super init];
	mAddonType = KUndefinedScriptType;
	mSource = nil;
	return self;
}

- (id)initWithContentsOfFile:(NSString*)fileName
{
	[super init];
	
	NSFileWrapper* wrapper = [[NSFileWrapper alloc] initWithPath:fileName];
	NSDictionary* attrDict = [wrapper fileAttributes];
	if([[attrDict objectForKey:NSFileHFSTypeCode] isEqualTo:[NSNumber numberWithUnsignedLong:NSHFSTypeCodeFromFileType(@"'APPL'")]]){
		mAddonType = KApplicationType;
		mSource = [[NSString alloc] initWithString:fileName];
	}else if([[fileName pathExtension] isEqualToString:@"applescript"]){
		mAddonType = KAppleScriptType;
		NSString* fileContents = [NSString stringWithContentsOfFile:fileName];
		NSString* scriptString = [KStringUtils replaceString:fileContents search:@"\n" replace:@"\r"];
		mSource = [scriptString retain];
	}else if([[fileName pathExtension] isEqualToString:@"rb"]
			|| [[fileName pathExtension] isEqualToString:@"pl"]
			|| [[fileName pathExtension] isEqualToString:@"php"]
			|| [[fileName pathExtension] isEqualToString:@"sh"]){
		mAddonType = KShellScriptType;
		mSource = [[NSString alloc] initWithString:fileName];
	}else{
		// do nothing...
	}
	
	[wrapper release];
	
	return self;
}



+ (id)addon
{
	return [[[self alloc] init] autorelease];
}

+ (id)addonWithContentsOfFile:(NSString*)fileName
{
	return [[[self alloc] initWithContentsOfFile:fileName] autorelease];
}



// check the object.
- (BOOL)isValid
{
	if(mAddonType != KUndefinedScriptType && mSource){ return YES; }
	return NO;
}

// execute script.
// if an error has occured, return nil; owtherwise return the edited string.
// type will be filled the editing type. if you pass it nil, it'll be ignored.
- (NSString*)runAddonFor:(NSString*)target 
		range:(NSRange)range 
		type:(KAddonEditingType*)type
		path:(NSString*)fileName
{
	if(mAddonType == KApplicationType){
		//NSLog(mSource);
		system([[NSString stringWithFormat:@"'%@' &", mSource] cString]);
	}else if(mAddonType == KAppleScriptType){
		NSAppleScript* appleScript = [[NSAppleScript alloc] initWithSource:mSource];
		NSDictionary* dict;
		[appleScript executeAndReturnError:&dict];
		[appleScript release];
		if(type){ *type = KNoTouch; }
		return [NSString string];
		
	}else if(mAddonType == KShellScriptType){
		NSString* partialTextFileName = [self makeTmpFile:[target substringWithRange:range]];
		NSString* wholeTextFileName = [self makeTmpFile:target];
		NSString* settingTextFileName = [self makeTmpFile:@"FUNC=INSERT"];
		
		putenv([[NSString stringWithFormat:@"TMPFILE=%@", partialTextFileName] cString]);
		putenv([[NSString stringWithFormat:@"TMPFILE_PARTIAL=%@", partialTextFileName] cString]);
		putenv([[NSString stringWithFormat:@"TMPFILE_WHOLE=%@", wholeTextFileName] cString]);
		putenv([[NSString stringWithFormat:@"TMPSETTING=%@", settingTextFileName] cString]);
		if(fileName){
			putenv([[NSString stringWithFormat:@"DOCUMENT_PATH=%@", fileName] cString]);
		}else{
			putenv([@"DOCUMENT_PATH=" cString]);
		}
		
		FILE* aFP = popen([[NSString stringWithFormat:@"'%@'", mSource] cString], "r");
		
		NSMutableData* muData = [NSMutableData dataWithCapacity:10000000];
		char aChar;
		
		while(YES){
			aChar = fgetc(aFP);
			if(aChar == EOF){ break; }
			[muData appendBytes:(void*)&aChar length:sizeof(char)];
		}
		
		pclose(aFP);
		
		NSString* currentFunc = [self readTmpFile:settingTextFileName];
		if(type){
			if([KStringUtils string:currentFunc startsWith:@"FUNC=INSERT"]){
				*type = KReplaceSelection;
			}else if([KStringUtils string:currentFunc startsWith:@"FUNC=REPLACEALL"]){
				*type = KReplaceAll;
			}else if([KStringUtils string:currentFunc startsWith:@"FUNC=NOTOUCH"]){
				*type = KNoTouch;
			}
		}
		
		[self removeTmpFile:partialTextFileName];
		[self removeTmpFile:wholeTextFileName];
		[self removeTmpFile:settingTextFileName];
		
		NSString* resString = [[NSString alloc] initWithData:muData encoding:NSUTF8StringEncoding];
		return [resString autorelease];
		
	}else{
		// do nothing.
	}
	return nil;
}

// accessor methods.
- (KAddonContentType)addonType { return mAddonType; }
- (void)setAddonType:(KAddonContentType)type { mAddonType = type; }
- (id)source { return mSource; }

- (void)setSource:(id)source {
	[mSource release];
	mSource = [source retain];
}

// internal use.
- (NSString*)makeTmpFile:(NSString*)string
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSString* convString = string;
	convString = [KStringUtils replaceString:convString search:@"\r\n" replace:@"\n"];
	convString = [KStringUtils replaceString:convString search:@"\r" replace:@"\n"];
	NSData* data = [convString dataUsingEncoding:NSUTF8StringEncoding];
	
	unsigned i;
	NSString* fileName;

	for(i = 0; i < 10000; i++){
		fileName = [NSString stringWithFormat:@"/tmp/kedit-%d.txt", i];
		if(![manager fileExistsAtPath:fileName]){
			[manager createFileAtPath:fileName contents:data attributes:nil];
			break;
		}
	}
	
	return fileName;
}

- (NSString*)readTmpFile:(NSString*)fileName
{
	NSData* data = [NSData dataWithContentsOfFile:fileName];
	NSString* string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
	return [string autorelease];
}

- (void)removeTmpFile:(NSString*)fileName
{
	NSFileManager* manager = [NSFileManager defaultManager];
	[manager removeFileAtPath:fileName handler:nil];
}

- (void)dealloc
{
	[mSource release];
}







@end
