#import "KApplicationDel.h"
#import "MyDocument.h"
#import "KSearchServer.h"
#import "KPrefsServer.h"
#import "KMenuUtils.h"
#import "KWebBrowser.h"

@implementation KApplicationDel

- (id)init
{
	[super init];
	
	mPrefsServer = [[KPrefsServer alloc] init];
	mSearchServer = nil;
	
	NSMenuItem* menuItem;
	
	//Bookmarks.
	mBookmarksMenu = [[NSMenu alloc] initWithTitle:@"bookmarks"];
	//[KMenuUtils attachURLMenuToMenu:mBookmarksMenu selector:@selector(insertMenuURL:)];
	id browser = [KWebBrowser browserWithBrowserType:KWebBrowser_Safari];
	menuItem = [[NSMenuItem alloc] initWithTitle:@"Safari" action:nil keyEquivalent:@""];
	[mBookmarksMenu addItem:[menuItem autorelease]];
	[menuItem setSubmenu:[browser bookmarksMenuWithSelector:@selector(insertMenuURL:)]];
	
	[browser setWebBrowserType:KWebBrowser_IE];
	menuItem = [[NSMenuItem alloc] initWithTitle:@"IE" action:nil keyEquivalent:@""];
	[mBookmarksMenu addItem:[menuItem autorelease]];
	[menuItem setSubmenu:[browser bookmarksMenuWithSelector:@selector(insertMenuURL:)]];
  
	//Tags.
  mHTMLTagsMenu = [[NSMenu alloc] initWithTitle:@"tags"];
  [KMenuUtils attachTagMenuToMenu:mHTMLTagsMenu selector:@selector(insertMenuTag:)];
  
	//Tools.
  mToolsMenu = [[NSMenu alloc] initWithTitle:@"Tools"];
  [KMenuUtils attachScriptsMenuToMenu:mToolsMenu selector:@selector(executeScript:)];

	return self;
}

- (void)dealloc
{
	NSLog(@"KApplicationDel : dealloc");
	[mSearchServer release];
	[mPrefsServer release];
  
	[mBookmarksMenu release];
  [mHTMLTagsMenu release];
  [mToolsMenu release];
	
	[super dealloc];
}

// MainMenu loaded.
- (void)awakeFromNib
{
	//[self constructMenus];
  NSMenu* mainMenu = [NSApp mainMenu];
  id menuItem = [KMenuUtils searchMenuItemWithTag:KRNMenuItemTag_HTMLTag targetMenu:mainMenu];
  [menuItem setSubmenu:[self htmlTagsMenu]];
  
  menuItem = [KMenuUtils searchMenuItemWithTag:KRNMenuItemTag_Bookmarks targetMenu:mainMenu];
  [menuItem setSubmenu:[self bookmarksMenu]];
  
  menuItem = [KMenuUtils searchMenuItemWithTag:KRNMenuItemTag_Tools targetMenu:mainMenu];
  [menuItem setSubmenu:[self toolsMenu]];
  
  // setup a popup for textview.
	menuItem = [myPopupMenuForTextView addItemWithTitle:NSLocalizedString(@"HTML Tags", @"") action:nil keyEquivalent:@""];
  [menuItem setSubmenu:[self htmlTagsMenu]];
  menuItem = [myPopupMenuForTextView addItemWithTitle:NSLocalizedString(@"URLs", @"") action:nil keyEquivalent:@""];
  [menuItem setSubmenu:[self bookmarksMenu]];
  menuItem = [myPopupMenuForTextView addItemWithTitle:NSLocalizedString(@"Tools", @"") action:nil keyEquivalent:@""];
  [menuItem setSubmenu:[self toolsMenu]];

}

- (BOOL)respondsToSelector:(SEL)aSelector
{
  if (aSelector == @selector(openUserDefinedTagFile:) && ![[NSApp delegate] userTagFileExists]){
    return NO;
  }
  return [super respondsToSelector:aSelector];
}

- (void)applicationDidFinishLaunching:(NSNotification*)notification
{
	//NSFileManager* manager = [NSFileManager defaultManager];
	//[manager createFileAtPath:@"/Users/karino/index.html" contents:[NSData data] attributes:nil];
	// do nothing.
}


- (void)showGotoLineWindow
{
	[myGotoLineWindow makeKeyAndOrderFront:self];
}

- (void)hideGotoLineWindow
{
	[myGotoLineWindow orderOut:self];
	[myGotoLineTextField setStringValue:@""];
}


- (id)searchServer
{
	if(mSearchServer == nil){
		mSearchServer = [[KSearchServer alloc] init];
	}
	return mSearchServer;
}

- (id)prefsServer
{
	return mPrefsServer;
}

- (id)popupMenuForTextView { return myPopupMenuForTextView; }

- (id)bookmarksMenu { return [mBookmarksMenu copyWithZone:nil]; }
- (id)htmlTagsMenu { return [mHTMLTagsMenu copyWithZone:nil]; }
- (id)toolsMenu { return [mToolsMenu copyWithZone:nil]; }

- (BOOL)userTagFileExists
{
  NSFileManager* manager = [NSFileManager defaultManager];
  NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
  NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
  //NSLog([NSString stringWithFormat:@"userTagFileExists - %d", [manager fileExistsAtPath:tagFileName]]);
  return [manager fileExistsAtPath:tagFileName];
}

- (IBAction)openUserDefinedTagFile:(id)sender
{
	NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
	NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
	NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
	
	BOOL wasOpened = [workspace openFile:tagFileName];
	
	if(!wasOpened){
		NSLog(@"KApplicationDel.m : openUserDefinedTagFile: - no such file.");
	}
}

@end



@implementation NSApplication (ScriptSupport)

- (void)handleCreateCommand:(NSScriptCommand*)command
{
	[[NSDocumentController sharedDocumentController] newDocument:nil];
}


@end


