#import "KPrefsServer.h"
#import "KColorUtils.h"
#import "KMenuUtils.h"
#import "KApplicationDel.h"
#import "KWebBrowser.h"

@implementation KPrefsServer

- (id)init
{
	NSString* fontName = [[NSFont userFixedPitchFontOfSize:0.0] fontName];
	id defaults = [NSUserDefaults standardUserDefaults];
	
	[defaults setBoolIfNeeded:YES forKey:KRNAutoIndent];
	[defaults setBoolIfNeeded:YES forKey:KRNWordWrap];
	[defaults setBoolIfNeeded:YES forKey:KRNDetectTextType];
	[defaults setBoolIfNeeded:YES forKey:KRNDetectCodeType];
	[defaults setBoolIfNeeded:YES forKey:KRNStoreTextType];
	[defaults setBoolIfNeeded:NO forKey:KRNAssignOnlyMultiStroke];
	[defaults setBoolIfNeeded:YES forKey:KRNUseXHTMLSyntax];
	[defaults setBoolIfNeeded:NO forKey:KRNAlwaysMakesANewWindow];
	
	[defaults setIntegerIfNeeded:(int)KKeybind_KEditOriginal forKey:KRNKeybindType];
	[defaults setIntegerIfNeeded:(int)KWebBrowser_Safari forKey:KRNPrefferedBrowserType];
	[defaults setIntegerIfNeeded:(int)KDocumentReturnCode_CR forKey:KRNDocumentReturnCodeType];
	[defaults setIntegerIfNeeded:(int)KDocumentCharacterCode_SJIS forKey:KRNDocumentCharacterCodeType];
	[defaults setIntegerIfNeeded:(int)KDocumentTextFileFormat_Plain forKey:KRNDocumentTextFileFormatType];
  
	[defaults setIntegerIfNeeded:60 forKey:KRNWrapLineWithReturnCodeWidth];
	
	[defaults setFloatIfNeeded:12.0 forKey:KRNDocumentFontSize];
	
	[defaults setObjectIfNeeded:fontName forKey:KRNDocumentFontName];
	
	[defaults synchronize];
	
	return self;
}

- (void)dealloc
{
	[super dealloc];
}


- (void)showPreferenceWindow
{
	if(myWindow == nil){
		[self loadPreferenceWindow];
	}
	[myWindow makeKeyAndOrderFront:self];
}

- (void)loadPreferenceWindow
{
	[NSBundle loadNibNamed:@"PreferenceWindow" owner:self];
}


// Panel of PreferenceWindow.nib : delegate method
- (void)awakeFromNib
{
	[myTabView selectFirstTabViewItem:self];
}

- (BOOL)windowShouldClose:(id)sender
{
	if(sender == myWindow){
		[myWindow orderOut:self];
		return NO;
	}
	return YES;
}


////////
// Actions
////////


- (IBAction)okButtonPressed:(id)sender
{
	[myWindow orderOut:self];
	[[NSUserDefaultsController sharedUserDefaultsController] save:self];
}

- (IBAction)cancelButtonPressed:(id)sender
{
	[myWindow orderOut:self];
	[[NSUserDefaultsController sharedUserDefaultsController] revert:self];
}

- (IBAction)fontSetButtonPressed:(id)sender
{
  id fontPanel = [NSFontPanel sharedFontPanel];
  [fontPanel setDelegate:self];
  id defaults = [NSUserDefaults standardUserDefaults];
  id font = [NSFont fontWithName:[defaults objectForKey:KRNDocumentFontName] size:[defaults floatForKey:KRNDocumentFontSize]];
  [fontPanel setPanelFont:font isMultiple:NO];
  [fontPanel makeKeyAndOrderFront:self];
}

- (IBAction)changeFont:(id)sender
{
  id defaults = [NSUserDefaults standardUserDefaults];
  id oldFont = [NSFont fontWithName:[defaults objectForKey:KRNDocumentFontName] size:[defaults floatForKey:KRNDocumentFontSize]];
  id newFont = [sender convertFont:oldFont];
  [defaults setObject:[newFont fontName] forKey:KRNDocumentFontName];
  [defaults setFloat:[newFont pointSize] forKey:KRNDocumentFontSize];
}


@end



// NSMutableDictionary.
@implementation NSMutableDictionary (Preferences)

- (BOOL)boolForKey:(id)key { return [[self objectForKey:key] boolValue]; }
- (void)setBool:(BOOL)flag forKey:(id)key { [self setObject:[NSNumber numberWithBool:flag] forKey:key]; }
- (float)floatForKey:(id)key { return [[self objectForKey:key] floatValue]; }
- (void)setFloat:(float)number forKey:(id)key { [self setObject:[NSNumber numberWithFloat:number] forKey:key]; }
- (int)intForKey:(id)key { return [[self objectForKey:key] intValue]; }
- (void)setInt:(int)number forKey:(id)key { [self setObject:[NSNumber numberWithInt:number] forKey:key]; }
- (rgb_color)colorForKey:(id)key { return *(rgb_color*)[[self objectForKey:key] bytes]; }
- (void)setColor:(rgb_color)color forKey:(id)key { [self setObject:[NSData dataWithBytes:&color length:sizeof(rgb_color)] forKey:key]; }

@end

@implementation NSUserDefaults (Preferences)

- (void)setBoolIfNeeded:(BOOL)value forKey:(NSString*)key
{
  if(![self objectForKey:key]){ [self setBool:value forKey:key]; }
}

- (void)setFloatIfNeeded:(float)value forKey:(NSString*)key
{
  if(![self objectForKey:key]){ [self setFloat:value forKey:key]; }
}

- (void)setIntegerIfNeeded:(int)value forKey:(NSString*)key
{
  if(![self objectForKey:key]){ [self setInteger:value forKey:key]; }
}

- (void)setObjectIfNeeded:(id)value forKey:(NSString*)key
{
  if(![self objectForKey:key]){ [self setObject:value forKey:key]; }
}

@end











