#import "KSearchServer.h"
#import "KApplicationDel.h"
#import "MyDocument.h"

@implementation KSearchServer


- (id)init
{
	mSearchString = [[NSString alloc] initWithString:@""];
	mReplaceString = [[NSString alloc] initWithString:@""];
	
	mIgnoreCase = YES;
	mUseRegex = NO;
	
	mSearchCaller = nil;
	
	[NSBundle loadNibNamed:@"SearchWindow" owner:self];
	// next, access the prefsServer to set initial value of data-members.
	return self;
}

- (void)awakeFromNib
{
	// set-up the search panel.
	//[myUseRegexCheckBox setEnabled:NO];
}

- (BOOL)windowShouldClose:(id)sender
{
	if (sender == mySearchWindow){
		[mySearchWindow orderOut:self]; 
		return NO;
	}
	return YES;
}

- (void)showSearchWindow:(id)sender
{
	mSearchCaller = sender;
	[mySearchWindow makeKeyAndOrderFront:self];
}

- (NSRange)searchString:(NSTextView*)textView direction:(KSearchDirection)direction
{
	
	NSString* currentString = [[textView textStorage] string];
	NSRange currentRange = [textView selectedRange];
	unsigned mask = 0;
	int startOffset, searchLength;
	
	if(mIgnoreCase){ mask = (mask | NSCaseInsensitiveSearch); }
	if(direction == KSearchBackward){
		mask = (mask | NSBackwardsSearch);
		startOffset = 0;
		searchLength = currentRange.location;
	}else{
		startOffset = currentRange.location + currentRange.length;
		searchLength =  [currentString length] - startOffset;
	}
	//NSLog([NSString stringWithFormat:@"start = %d, length = %d, ign = %d", startOffset, searchLength, mIgnoreCase]);
	
	return [currentString rangeOfString:mSearchString options:mask range:NSMakeRange(startOffset, searchLength)];
}

- (unsigned)replaceAll:(NSTextView*)textView
{
	NSMutableArray* stringArray = [[NSMutableArray alloc] initWithCapacity:1000];
	NSString* substring;
	NSString* string = [textView string];
	unsigned mask = 0;
	unsigned count = 0;
	NSRange currentRange = NSMakeRange(0, [string length]);
	NSRange bufRange;
	if(mIgnoreCase){ mask = (mask | NSCaseInsensitiveSearch); }
	
	while(YES){
		bufRange = [string rangeOfString:mSearchString options:mask range:currentRange];
		if(bufRange.location == NSNotFound){ break; }
		
		substring = [[NSString alloc] initWithString:[string substringWithRange:NSMakeRange(currentRange.location, bufRange.location - currentRange.location)]];
		[stringArray addObject:substring];
		[substring release];
		
		currentRange.location = NSMaxRange(bufRange);
		currentRange.length = [string length] - currentRange.location;
		count++;
	}
	
	unsigned newLength = ([mReplaceString length] - [mSearchString length]) * [stringArray count];
	
	NSMutableString* replaceString = [[NSMutableString alloc] initWithCapacity:newLength];
	int i;
	for(i = 0; i < [stringArray count]; i++){
		[replaceString appendString:[stringArray objectAtIndex:i]];
		[replaceString appendString:mReplaceString];
	}
	[replaceString appendString:[string substringWithRange:currentRange]];
	
	[textView setString:replaceString];
	
	[stringArray removeAllObjects];
	[stringArray release];
	
	return count;
}



////////
// Actions for Buttons.
////////

- (IBAction)searchButtonPressed:(id)sender
{
	//[self setButtonPressed:nil];
	[mySearchWindow orderOut:self];
	
	if(mSearchCaller != nil){
		[mSearchCaller doSearch:self direction:KSearchForward];
	}
}
- (IBAction)replaceAllButtonPressed:(id)sender
{
	[mySearchWindow orderOut:self];
	
	if(mSearchCaller != nil){
		[mSearchCaller doReplaceAll:self];
	}
}
- (IBAction)cancelButtonPressed:(id)sender
{
	[mySearchWindow orderOut:self];
}


////////
// Data-members
////////

- (void)setSearchString:(NSString*)searchString
{
  //NSLog([NSString stringWithFormat:@"search : %@", searchString]);
	
	[mSearchString release];
  if (searchString){
    mSearchString = [[NSString alloc] initWithString:searchString];
  }else{
    mSearchString = [[NSString alloc] initWithString:@""];
  }
	
	
}

- (void)setReplaceString:(NSString*)replaceString
{
	[mReplaceString release];
  if (replaceString){
    mReplaceString = [[NSString alloc] initWithString:replaceString];
  }else{
    mReplaceString = [[NSString alloc] initWithString:@""];
  }
	
	//NSLog([NSString stringWithFormat:@"replace : %@", mReplaceString]);
}

- (NSString*)searchString { return mSearchString; }
- (NSString*)replaceString { return mReplaceString; }

- (void)setIgnoreCase:(BOOL)ignoreCase { mIgnoreCase = ignoreCase; }
- (BOOL)ignoreCase { return mIgnoreCase; }
- (void)setUseRegex:(BOOL)useRegex { mUseRegex = useRegex; }
- (BOOL)useRegex { return mUseRegex; }

@end
