//
//  KSettingFile.m
//  KEdit
//
//  Created by Karino, Masatsugu on Thu May 22 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KSettingFile.h"
#import "KStringUtils.h"
#import "KPrefsServer.h"

@implementation KSettingFile

- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type
{
	mBundlePath = mUserPath = nil;
	
	if(!fileName || !type){ return nil; }
	
	NSFileManager* manager = [NSFileManager defaultManager];
	NSBundle* bundle = [NSBundle mainBundle];
	mBundlePath = [bundle pathForResource:fileName ofType:type];
	if(!mBundlePath){ return nil; }
	[mBundlePath retain];
	
	mUserPath = [NSString stringWithFormat:@"%@/%@/%@/%@.%@", 
					NSHomeDirectory(), KPreferenceDirectory, 
					KSettingsDirectoryName, fileName, type];
	if([manager fileExistsAtPath:mUserPath]){
		[mUserPath retain];
	}else{
		mUserPath = nil;
	}
	
	mFileName = [fileName retain];

	[super init];
	[self analyze];
	
	return self;
}

+ (id)settingsWithFileOfName:(NSString*)fileName ofType:(NSString*)type
{
	return [[[self alloc] initWithFileOfName:fileName ofType:type] autorelease];
}

- (BOOL)analyze
{
	NSLog(@"analize NO!");
	return NO; // do nothing.
}

- (NSString*)fileName
{
	return mFileName;
}

- (void)dealloc
{
	[mBundlePath release];
	[mUserPath release];
	[mFileName release];
}

@end



@implementation KModeSettings

- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type
{
	mSettings = nil;
	[super initWithFileOfName:fileName ofType:type];
	return self;
}

- (NSDictionary*)settings
{
	return mSettings;
}

- (id)settingForKey:(NSString*)key
{
	return [mSettings objectForKey:key];
}

- (NSColor*)colorForKey:(NSString*)key
{
	// string -> color
	return [KStringUtils colorFromString:[mSettings objectForKey:key]];
}

// preset colors
- (NSColor*)backgroundColor
{
	return [self colorForKey:KModeSettingsDefault_BackgroundColor];
}
- (NSColor*)baseColor
{
	return [self colorForKey:KModeSettingsDefault_BaseStringColor];
}
- (NSColor*)commentColor
{
	return [self colorForKey:KModeSettingsDefault_CommentColor];
}
- (NSColor*)keywordColor
{
	return [self colorForKey:KModeSettingsDefault_KeywordColor];
}
- (NSColor*)doublequoteColor
{
	return [self colorForKey:KModeSettingsDefault_DoubleQuoteColor];
}
- (NSColor*)singlequoteColor
{
	return [self colorForKey:KModeSettingsDefault_SingleQuoteColor];
}
- (NSColor*)variableColor
{
	return [self colorForKey:KModeSettingsDefault_VariableColor];
}
- (NSColor*)groupColor
{
	return [self colorForKey:KModeSettingsDefault_GroupColor];
}
- (NSColor*)itemColor
{
	return [self colorForKey:KModeSettingsDefault_ItemColor];
}
- (NSColor*)bracketColor
{
	return [self colorForKey:KModeSettingsDefault_BracketColor];
}
- (NSColor*)indexColor
{
	return [self colorForKey:KModeSettingsDefault_IndexColor];
}
- (NSColor*)tagColor
{
	return [self colorForKey:KModeSettingsDefault_TagColor];
}
- (NSColor*)refColor
{
	return [self colorForKey:KModeSettingsDefault_RefColor];
}
// floats
- (float)fontSize
{
	return [[self settingForKey:KModeSettingsDefault_FontSize] floatValue];
}
// bools
- (BOOL)isWordWrapped
{
	NSString* string = [self settingForKey:KModeSettingsDefault_IsWordWrapped];
	if([[string lowercaseString] isEqualToString:@"yes"]){
		return YES;
	}else if([[string lowercaseString] isEqualToString:@"no"]){
		return NO;
	}else{
		NSLog(@"KSettingFile::isWordWrapped - neither yes nor no.");
		return YES;
	}
}


- (BOOL)analyze
{
	//NSLog(@"analize YES!");
	NSMutableDictionary* defDict = [NSMutableDictionary dictionaryWithDictionary:[KStringUtils settingsFromString:[NSString stringWithContentsOfFile:mBundlePath]]];
	if(!defDict){ return NO; }
	
	if(mUserPath){
		NSDictionary* userDict = [KStringUtils settingsFromString:[NSString stringWithContentsOfFile:mUserPath]];
		[defDict addEntriesFromDictionary:userDict];
	}
	mSettings = [defDict retain];
	//NSLog(@"analysis done.");
	return YES; // do nothing.
}

- (void)dealloc
{
	[mSettings release];
	[super dealloc];
}

- (void)printToStream
{
	NSEnumerator *enumerator = [mSettings keyEnumerator];
	id key;
	while ((key = [enumerator nextObject])) {
		NSLog([NSString stringWithFormat:@"[%@] %@", key, [mSettings objectForKey:key]]);
	}
}

@end



@implementation KKeywords

- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type
{
	mKeywords = nil;
	[super initWithFileOfName:fileName ofType:type];
	return self;
}

- (id)initWithFileOfName:(NSString*)fileName ofType:(NSString*)type withSeparators:(NSString*)characters
{
	[self initWithFileOfName:fileName ofType:type];
	mSeparators = [characters retain];
	return self;
}

+ (id)keywordsWithFileOfName:(NSString*)fileName ofType:(NSString*)type withSeparators:(NSString*)characters
{
	return [[[self alloc] initWithFileOfName:fileName ofType:type withSeparators:characters] autorelease];
}

- (NSArray*)keywords
{
	return mKeywords;
}

- (BOOL)analyze
{
	NSString* wordString;
	if(mUserPath){ wordString = [NSString stringWithContentsOfFile:mUserPath]; }
	else{ wordString = [NSString stringWithContentsOfFile:mBundlePath]; }
	if(!wordString) { return NO; }
	
	NSArray* array = [KStringUtils string:wordString componentsSeparatedByCharacters:@", \t\r\n"];
	if(!array) { return NO; }
	
	mKeywords = [array retain];
	return YES;
}

- (void)dealloc
{
	[mKeywords release];
	[super dealloc];
}

@end





















