//
//  KWebBrowser.m
//  KEdit
//
//  Created by Karino, Masatsugu on Thu Dec 04 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "KWebBrowser.h"

#import "KStringUtils.h"

@implementation KWebBrowser

- (id)initWithBrowserType:(KWebBrowserType)type
{
  [super init];
	mWebBrowserType = type;
	return self;
}

+ (id)browserWithBrowserType:(KWebBrowserType)type
{
  return [[[self alloc] initWithBrowserType:type] autorelease];
}

- (void)openDocument:(NSDocument*)document requireNewWindow:(BOOL)requireNewWindow
{
  NSFileManager* manager = [NSFileManager defaultManager];
	NSArray* components = [manager componentsToDisplayForPath:[document fileName]];
	NSString* newPath = [components componentsJoinedByString:@":"];
	NSString* newURL = [components componentsJoinedByString:@"/"];
  //id defaults = [NSUserDefaults standardUserDefaults];
	NSString* scriptString;
	
  if (mWebBrowserType == KWebBrowser_IE){
    if (requireNewWindow){
			scriptString = [NSString stringWithFormat:
				@"tell application \"Internet Explorer\"\r"
				@"activate\r"
				@"open alias \"%@\"\r"
				@"end tell\r"
				, newPath];
			
		}else{ // loaed into the frontmost window of IE.
			scriptString = [NSString stringWithFormat:
				@"tell application \"Internet Explorer\"\r"
				@"activate\r"
				@"ShowFile alias \"%@\" URL \"file://localhost/Volumes/%@\"\r"
				@"end tell\r"
				, newPath, newURL];
		}
	
		NSAppleScript* appleScript = [[NSAppleScript alloc] initWithSource:scriptString];
		NSDictionary* dict;
		[appleScript executeAndReturnError:&dict];
		[appleScript release];
		
  }else if (mWebBrowserType == KWebBrowser_Safari){
		/* Currently Safari can't be controlled with AppleScript enoughly. */
		/* The routines below don't act properly. */
    if (requireNewWindow){
			scriptString = [NSString stringWithFormat:
				@"tell application \"Safari\"\r"
				@"activate\r"
				@"open alias \"%@\"\r"
				@"end tell\r"
				, newPath];
		}else{
			/* this code below was written by Mr. Hetima. Thanx! */
			NSURL* tempURL=[[[NSURL alloc]initFileURLWithPath:[document fileName]]autorelease];
			scriptString = [NSString stringWithFormat:
				@"tell application \"Safari\"\r"
				@"activate\r"
				@"reopen\r"
				@"set URL of document 1 to \"%@\"\r"
				@"end tell\r"
				, [tempURL absoluteString]];
		}
		NSAppleScript* appleScript = [[NSAppleScript alloc] initWithSource:scriptString];
		NSDictionary* dict;
		[appleScript executeAndReturnError:&dict];
		[appleScript release];
	}
}

- (NSMenu*)bookmarksMenuWithSelector:(SEL)selector
{
  // the format of dictionaries of the represented object is below;
	//   title : NSString : page title
	//   url : NSString : page url
	
  NSMenu* menu = [[NSMenu alloc] initWithTitle:@"Bookmarks"];
  switch((int)mWebBrowserType){
	  case KWebBrowser_Safari:[self constructURLMenuWithSafariBookmarks:menu selector:selector]; break;
	  case KWebBrowser_IE:[self constructURLMenuWithIEBookmarks:menu selector:selector]; break;
	}
	return [menu autorelease];
}

- (KWebBrowserType)webBrowserType
{
  return mWebBrowserType;
}

- (void)setWebBrowserType:(KWebBrowserType)type
{
  mWebBrowserType = type;
}





- (void)constructURLMenuWithSafariBookmarks:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSString* SafariBookmarkFileName = [NSHomeDirectory() stringByAppendingPathComponent:KSafariBookmarkFileName];
	NSDictionary* SafariBookmarkContents = [NSDictionary dictionaryWithContentsOfFile:SafariBookmarkFileName];
	if(!SafariBookmarkContents){ return; }
	NSArray* array = [SafariBookmarkContents objectForKey:@"Children"];
	if(!array){ return; }
	
	[self constructURLMenuWithSafariBookmarksWithArray:targetMenu selector:selector array:array];
}

- (void)constructURLMenuWithSafariBookmarksWithArray:(NSMenu*)targetMenu selector:(SEL)selector array:(NSArray*)array
{
	unsigned i;
	NSDictionary* dict;
	NSArray* children = nil;
	NSMenu* menu;
	id menuItem;
	NSString* title, *url;
	
	for(i = 0; i < [array count]; i++){
		dict = [array objectAtIndex:i];
		if([[dict objectForKey:@"WebBookmarkType"] isEqualToString:@"WebBookmarkTypeList"]){
			if([[dict objectForKey:@"Title"] isEqualToString:@"Imported IE Favorites"]){ continue; }
			children = [dict objectForKey:@"Children"];
			if(children){
				menuItem = [[NSMenuItem alloc] initWithTitle:[dict objectForKey:@"Title"] action:nil keyEquivalent:@""];
				menu = [[NSMenu alloc] initWithTitle:@""];
				[targetMenu addItem:[menuItem autorelease]];// autoreleased
				[menuItem setSubmenu:[menu autorelease]]; // autoreleased
				[self constructURLMenuWithSafariBookmarksWithArray:menu selector:selector array:children];
			}
		}else if([[dict objectForKey:@"WebBookmarkType"] isEqualToString:@"WebBookmarkTypeLeaf"]){
			title = [[dict objectForKey:@"URIDictionary"] objectForKey:@"title"];
			url = [dict objectForKey:@"URLString"];
			menuItem = [[NSMenuItem alloc] initWithTitle:title action:selector keyEquivalent:@""];
			[targetMenu addItem:[menuItem autorelease]];//autoreleased
			[menuItem setRepresentedObject:[NSDictionary dictionaryWithObjectsAndKeys:title, @"title", url, @"url", nil]];
		}
	}
}


- (void)constructURLMenuWithIEBookmarks:(NSMenu*)targetMenu selector:(SEL)selector
{	
	NSString* IEBookmarkFileName = [NSHomeDirectory() stringByAppendingPathComponent:KIEBookmarkFileName];
	NSString* IEBookmarkContents = [NSString stringWithContentsOfFile:IEBookmarkFileName];
	
	if(IEBookmarkContents){
		NSArray* lineArray = [IEBookmarkContents componentsSeparatedByString:@"\r"];//[KStringUtils string:IEBookmarkContents separatedWith:@"\r"];
		NSArray* bufferArray;
		NSString* title, *url;
		int i;
		NSMenu* currentMenu = targetMenu;
		id currentMenuItem;
		
		for(i = 0; i < [lineArray count]; i++){
			if([KStringUtils string:[lineArray objectAtIndex:i] contains:@"</H3>"]){
				bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"\">" andWith:@"</H3>"];
				if(!bufferArray || ![bufferArray count]){ continue; } // fixed with Mr. YOUsuke's advice. Thanx!

				currentMenuItem = [currentMenu addItemWithTitle:[bufferArray objectAtIndex:0] action:nil keyEquivalent:@""];
				currentMenu = [[NSMenu alloc] initWithTitle:@""];
				[currentMenuItem setSubmenu:[currentMenu autorelease]];//autoreleased
				continue;
			}else if([KStringUtils string:[lineArray objectAtIndex:i] contains:@"</DL>"]){
				if(currentMenu == targetMenu) { break; } // not allowed to touch the super menu of targetMenu.
				currentMenu = [currentMenu supermenu];
				continue;
			}
			bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"A HREF=\"" andWith:@"\""];
			if([bufferArray count] > 0){
				url = [bufferArray objectAtIndex:0];
				bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"\">" andWith:@"</A>"];
				if([bufferArray count] > 0){
					title = [bufferArray objectAtIndex:0];
					currentMenuItem = [[NSMenuItem alloc] initWithTitle:title action:selector keyEquivalent:@""];
					[currentMenuItem setRepresentedObject:[NSDictionary dictionaryWithObjectsAndKeys:title, @"title", url, @"url", nil]];
					[currentMenu addItem:[currentMenuItem autorelease]];//autoreleased
				}
			}
		}
	}
}


@end
