//
//  MyDocument.m
//  KEdit
//
//  Created by Karino, Masatsugu on Tue Feb 18 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//
//#import <Carbon/Carbon.h>
#import <Carbon/Carbon.h>

#import "MyDocument.h"
#import "KApplicationDel.h"
#import "KMenuUtils.h"
#import "KStringUtils.h"
#import "KTextView.h"
#import "KRangeObj.h"
#import "KColorSyntaxServer.h"
#import "KSettingFile.h"
#import "KAddon.h"
#import "KLineRuler.h"
#import "KWebBrowser.h"

@implementation MyDocument

- (id)init
{
  [super init];
  if (self) {
    // Add your subclass-specific initialization here.
    // If an error occurs here, send a [self dealloc] message and return nil.
		
    //set initial data to data-members.
		id defaults = [NSUserDefaults standardUserDefaults];
		
		mBufString = nil;
		mReturnCodeType = [defaults integerForKey:KRNDocumentReturnCodeType];
		mDocumentEncoding = [defaults integerForKey:KRNDocumentCharacterCodeType];
		mTextType = [defaults integerForKey:KRNDocumentTextFileFormatType];
		mSelectedRange = NSMakeRange(0, 0);
		
		mWordWrap = [defaults boolForKey:KRNWordWrap];
		mAutoIndent = [defaults boolForKey:KRNAutoIndent];
		mCurrentLineNumber = 1;
    mTextViewFontName = [[NSString alloc] initWithString:[defaults objectForKey:KRNDocumentFontName]];
    mTextViewFontSize = [defaults floatForKey:KRNDocumentFontSize];
		
		mColorSyntaxServer = nil;
  }
  return self;
}

- (void)dealloc
{
	[mColorSyntaxServer release];
	[mTextViewFontName release];
	
	[super dealloc];
}

- (BOOL)respondsToSelector:(SEL)aSelector
{
  if ( (aSelector == @selector(saveDocument:) && ![self isDocumentEdited])
      || (aSelector == @selector(revertDocumentToSaved:) && (![self fileName] || ![self isDocumentEdited]))
      || (aSelector == @selector(openWithBrowser:) && ![self fileName]) ){
    return NO;
  }
  return [super respondsToSelector:aSelector];
}


- (id)initWithContentsOfFile:(NSString*)fileName ofType:(NSString*)docType
{
	[super initWithContentsOfFile:fileName ofType:docType];
	return self;
}


- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"MyDocument";
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    // Insert code here to write your document from the given data.  You can also choose to override -fileWrapperRepresentationOfType: or -writeToFile:ofType: instead.
    
	NSString* string = [myTextView string];
	string = [KStringUtils convertReturnsFromLF:string to:mReturnCodeType];
	NSData* aData = [string dataUsingEncoding:[self documentEncoding]];
	//aData = [KTextUtils convertReturnCode:aData from:mReturnCodeType to:mUserDefinedReturnCodeType];
	
	//[[NSWorkspace sharedWorkspace] setFinderInfo:[self fileName] fileType:@"TEXT" creator:@"KEDI"];
	
	return aData;
	
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
    // Insert code here to read your document from the given data.  You can also choose to override -loadFileWrapperRepresentation:ofType: or -readFromFile:ofType: instead.
    
	//set data-members.
	if([data length]){
		mDocumentEncoding = [KTextUtils detectCharacterCodeType:data];
	}
	
	NSString* buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	if(!buf){
    id defaults = [NSUserDefaults standardUserDefaults];
		mDocumentEncoding = [defaults integerForKey:KRNDocumentCharacterCodeType];//NSShiftJISStringEncoding;
		buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	}
	
	mBufString = [[KStringUtils convertReturnsToLF:buf from:&mReturnCodeType] retain];
	[buf release];

	
	[self setTextTypeWithTypeName:aType];
	
	return YES;
}

// The last thing to do for the Window.
- (void)windowControllerDidLoadNib:(NSWindowController*)windowController
{
	[super windowControllerDidLoadNib:windowController];
	[[myTextView window] makeFirstResponder:myTextView];
	
	if(mBufString == nil){ // new document.
		[myTextView setString:@""];
    
	}else{ // open document.
		[myTextView setString:mBufString];
		[myTextView setSelectedRange:NSMakeRange(0, 0)];
		[mBufString release];
		mBufString = nil;
	}
	
	[self setWordWrapped:mWordWrap];
	
	[self adjustDocumentSettings];
  //[myLineNumberView setStringValue:@"1"];
	
	// SET TEXTVIEW FRAME.
	NSRect textviewFrame = [myTextView frame];
	textviewFrame.origin = NSMakePoint(textviewFrame.origin.x - 1, textviewFrame.origin.y - 1);
	textviewFrame.size = NSMakeSize(textviewFrame.size.width + 2, textviewFrame.size.height + 2);	
	[myTextView setFrame:textviewFrame];
  
  // ?Line Number View.
  //id ruler = [[KLineRuler alloc] initWithTextView:myTextView andScrollView:myScrollView];
  //[myScrollView setVerticalRulerView:ruler];
  //[myScrollView setRulersVisible:YES];
	
	// SET WINDOW FRAME. 
	NSRect windowFrame;
	NSRect screenFrame = [[NSScreen mainScreen] frame];
	windowFrame.size = NSMakeSize(600.0, 600.0);
	windowFrame.origin = NSMakePoint(30.0, screenFrame.size.height - windowFrame.size.height - 30.0);
	[myWindow setFrame:windowFrame display:YES];
  
	[self validateColorSyntax];
	
	//[myTextView registerForDraggedTypes:[NSArray arrayWithObjects:NSColorPboardType, NSFilenamesPboardType, nil]]; 
	//[[myTextView window] registerForDraggedTypes:[NSArray arrayWithObjects:NSColorPboardType, NSFilenamesPboardType, nil]]; 

}


////////
// from SearchServer
////////

- (void)doSearch:(id)sender direction:(KSearchDirection)direction
{
	id searchServer = [[NSApp delegate] searchServer];
	NSRange selectedRange = [searchServer searchString:myTextView direction:direction];
	if(selectedRange.location == NSNotFound){
		NSBeep();
	}else{
		[myTextView setSelectedRange:selectedRange];
		[myTextView scrollRangeToVisible:selectedRange];
	}
}

- (void)doReplaceAll:(id)sender
{
	id searchServer = [[NSApp delegate] searchServer];
	unsigned count = [searchServer replaceAll:myTextView];
	[self validateColorSyntax];
	[myTextView setSelectedRange:NSMakeRange(0, 0)];
  [self updateChangeCount:NSChangeDone];
	
	[self showReplacedAllDialog:count];
}


- (NSDictionary*)fileAttributesToWriteToFile:(NSString*)fullDocumentPath ofType:(NSString*)docType saveOperation:(NSSaveOperationType)saveOperationType
{
	NSNumber* creatorCode = nil;
	NSNumber* typeCode = nil;
  id defaults = [NSUserDefaults standardUserDefaults];
	NSDictionary* infoPlist = [[NSBundle mainBundle] infoDictionary];
	NSMutableDictionary* newList = [NSMutableDictionary dictionaryWithDictionary:[super fileAttributesToWriteToFile:fullDocumentPath ofType:docType saveOperation:saveOperationType]];
	NSString* creatorCodeString = [infoPlist objectForKey:@"CFBundleSignature"];
	
  if (creatorCodeString && [defaults boolForKey:KRNStoreTextType]){
		creatorCode = [NSNumber numberWithUnsignedLong:NSHFSTypeCodeFromFileType([NSString stringWithFormat:@"'%@'", creatorCodeString])];
		[newList setObject:creatorCode forKey:NSFileHFSCreatorCode];
		
		typeCode = [NSNumber numberWithUnsignedLong:NSHFSTypeCodeFromFileType(@"'TEXT'")];
		[newList setObject:typeCode forKey:NSFileHFSTypeCode];
	}
	return newList;
}



- (void)showReplacedAllDialog:(unsigned)count
{
	NSString* alertString = [NSString stringWithFormat:
					@"Replaced %d words",
					count];

	NSBeginAlertSheet(
            @"Replacement Information", 
            @"OK", 
            nil, 
            nil, 
            myWindow, 
            self, 
			@selector(showReplacedAllDialogEnded:returnCode:contextInfo:),
            nil, 
            nil, 
            alertString);
}

- (void)showReplacedAllDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	//Do nothing...
}




////////
// Hook Functions
////////

- (BOOL)revertToSavedFromFile:(NSString*)fileName ofType:(NSString*)type
{
	NSData* data = [NSData dataWithContentsOfFile:fileName];
	
	if(!data && [data length]){
		mDocumentEncoding = [KTextUtils detectCharacterCodeType:data];
	}
	
	NSString* buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	if(!buf){
		mDocumentEncoding = NSShiftJISStringEncoding;
		buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
		if(!buf){ return NO; }
	}
	
	NSString* convertedString = [[KStringUtils convertReturnsToLF:buf from:&mReturnCodeType] retain];
	[buf release];
	[myTextView setString:convertedString];
	[myTextView setSelectedRange:NSMakeRange(0, 0)];
	[self setTextTypeWithTypeName:type];
	[self validateColorSyntax];
	
	return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem*)anItem
{
	[super validateMenuItem:anItem];
	// currently do nothing.
	return YES;
}


// NSTextView's delegate method.
/*
- (BOOL)textView:(NSTextView*)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString
{
	//When deleting, get empty string @"".
	mSelectedRange = NSMakeRange(affectedCharRange.location, [replacementString length]);
	return YES;
}*/


- (void)textViewDidChangeSelection:(NSNotification *)aNotification
{
	NSRange selectedRange = [myTextView selectedRange];
	unsigned lineNum = [KStringUtils countLinesOfString:[myTextView string] range:NSMakeRange(0, selectedRange.location)];
	//[myLineNumberView setStringValue:[NSString stringWithFormat:@"%d", lineNum]];
	[self setCurrentLineNumber:lineNum];
}

// NSText's delegate method.
- (void)textDidChange:(NSNotification*)aNotification
{
	//This method will be called after textView:shouldChangetextInRange:replacementString:.
	//let document instance know the document had been edited.
	//[self validateColorSyntax:mSelectedRange];
	[self validateColorSyntax:[myTextView lastAffectedRange]];
	[self updateChangeCount:NSChangeDone];
}

// NSWindow's delegate method.
- (void)windowDidBecomeMain:(NSNotification *)aNotification
{
  // currently do nothing.
}



////////
// internal use
////////
- (void)setTextTypeWithTypeName:(NSString*)aType
{
	if([aType isEqualToString:@"text file"]){
		mTextType = KDocumentTextFileFormat_Plain;
	}else if([aType isEqualToString:@"HTML file"]){
		mTextType = KDocumentTextFileFormat_HTML;
	}else if([aType isEqualToString:@"css file"]){
		mTextType = KDocumentTextFileFormat_CSS;
	}else if([aType isEqualToString:@"php file"]){
		mTextType = KDocumentTextFileFormat_PHP;
	}else if([aType isEqualToString:@"perl file"]){
		mTextType = KDocumentTextFileFormat_Perl;
	}else if([aType isEqualToString:@"ruby file"]){
		mTextType = KDocumentTextFileFormat_Ruby;
	}
}



- (void)adjustDocumentSettings
{
	// set ColorSyntaxServer
	[mColorSyntaxServer release];
	mColorSyntaxServer = [[KColorSyntaxServer alloc] initWithTextStorage:[myTextView textStorage] ofType:mTextType];
	
	NSTextStorage* storage = [myTextView textStorage];
	NSRange validatedRange = NSMakeRange(0, [storage length]);
	
	// set Font size
	if([[mColorSyntaxServer settings] settingForKey:KModeSettingsDefault_FontSize]){ // dirty...
		float fontSize = [[mColorSyntaxServer settings] fontSize];
		if(fontSize){
			[self setTextViewFontSize:fontSize];
			NSFont* font = [NSFont fontWithName:mTextViewFontName size:fontSize];
			[storage addAttribute:NSFontAttributeName value:font range:validatedRange];
		}
	}
	
	// set word-wrap
	if([[mColorSyntaxServer settings] settingForKey:KModeSettingsDefault_IsWordWrapped]){ // dirty...
		BOOL isWordWrapped = [[mColorSyntaxServer settings] isWordWrapped];
		[self setWordWrapped:isWordWrapped];
	}
}

// for accessing data-members.
- (void)setWordWrapped:(BOOL)wordwrap
{
	NSRect frame = [myTextView frame];
	
	if(!wordwrap){
		frame.size.width = 5000;//kUnwrappedViewWidth;
		[myTextView setFrame:frame];
		[myScrollView setHasHorizontalScroller:YES];
		
	}else{
		NSRect scrollFrame = [[myScrollView horizontalScroller] frame];
		frame.size.width = scrollFrame.size.width;
		[myTextView setFrame:frame];
		[myScrollView setHasHorizontalScroller:NO];
	}
	mWordWrap = wordwrap;
}

- (void)setTextViewFontName:(NSString*)fontName
{
	[mTextViewFontName release];
	mTextViewFontName = [[NSString alloc] initWithString:fontName];
}

- (void)setTextViewFontSize:(float)fontSize
{
	mTextViewFontSize = fontSize;
}

- (void)setTextType:(KDocumentTextFileFormatType)type
{
	mTextType = type;
  [self adjustDocumentSettings];
	[self validateColorSyntax];
}

- (void)setDocumentEncoding:(KDocumentCharacterCodeType)type
{
  mDocumentEncoding = type;
}

- (void)setReturnCodeType:(KDocumentReturnCodeType)type
{
  mReturnCodeType = type;
}

- (KDocumentCharacterCodeType)documentEncoding
{
  return mDocumentEncoding;
}


- (void)setCurrentLineNumber:(unsigned int)num
{
  // no checks, and not change the real selection.
	mCurrentLineNumber = num;
}

- (KDocumentReturnCodeType)returnCodeType { return mReturnCodeType; }
- (KDocumentTextFileFormatType)textType { return mTextType; }
- (BOOL)isWordWrapped { return mWordWrap; }
- (BOOL)isAutoIndented { return mAutoIndent; }
- (void)setAutoIndented:(BOOL)autoindent { mAutoIndent = autoindent; }
- (NSFont*)textViewFont { return [NSFont fontWithName:mTextViewFontName size:mTextViewFontSize]; }
- (NSString*)textViewFontName { return mTextViewFontName; }
- (float)textViewFontSize { return mTextViewFontSize; }
- (unsigned int)currentLineNumber { return mCurrentLineNumber; }


@end



@implementation MyDocument (Actions)
////////
// Actions.
////////
- (IBAction)scrollToSelection:(id)sender
{
	[myTextView scrollRangeToVisible:[myTextView selectedRange]];
}

- (IBAction)toggleWordWrap:(id)sender
{
	[self setWordWrapped:!mWordWrap];
}

- (IBAction)toggleAutoIndent:(id)sender
{
	mAutoIndent = !mAutoIndent;
}

- (IBAction)toggleStoreWindowStatus:(id)sender
{
	mStoreWindowStatus = !mStoreWindowStatus;
}

- (IBAction)showPreferenceWindow:(id)sender
{
	[[[NSApp delegate] prefsServer] showPreferenceWindow];
}

- (IBAction)showColorPanel:(id)sender
{
	NSString* string = [[myTextView string] substringWithRange:[myTextView selectedRange]];
	NSColor* color = [KStringUtils colorFromString:string];
	
	NSColorPanel* panel = [NSColorPanel sharedColorPanel];
	//[panel setTarget:self];
	//[panel setAction:@selector(insertColorString:)];
	if(color){ [panel setColor:color]; }
	[panel makeKeyAndOrderFront:self];
}


- (IBAction)showFontPanel:(id)sender
{
  id fontPanel = [NSFontPanel sharedFontPanel];
  //[fontPanel setDelegate:self];
  [fontPanel setPanelFont:[self textViewFont] isMultiple:NO];
  //[fontPanel makeKeyAndOrderFront:self];
	[fontPanel makeKeyAndOrderFront:nil];
}

- (IBAction)changeFont:(id)sender
{
  id oldFont = [self textViewFont];
  id newFont = [sender convertFont:oldFont];
  [self setTextViewFontName:[newFont fontName]];
  [self setTextViewFontSize:[newFont pointSize]];
	//NSLog([NSString stringWithFormat:@"font size = %f", [self textViewFontSize]]);
  
  [self validateColorSyntax];
}

- (IBAction)insertColorString:(id)sender
{
	[myTextView insertColorString:[sender color]];
}

- (IBAction)redrawColors:(id)sender
{
	[self validateColorSyntax];
}

- (IBAction)openWithBrowser:(id)sender
{
	if(![self fileName]){ NSBeep(); return; }
	if([self isDocumentEdited]){ [self saveDocument:self]; }
	
	id defaults = [NSUserDefaults standardUserDefaults];
	id browser = [KWebBrowser browserWithBrowserType:[defaults integerForKey:KRNPrefferedBrowserType]];
	[browser openDocument:self requireNewWindow:[defaults boolForKey:KRNAlwaysMakesANewWindow]];
}

- (IBAction)executeScript:(id)sender
{
	NSString* path = [sender representedObject];
	if(GetCurrentKeyModifiers() & shiftKey){
		NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
		[workspace openFile:path withApplication:@"KEdit"];
	}else{
		KAddon* addon = [KAddon addonWithContentsOfFile:path];
		KAddonEditingType type;
		NSString* resString = [addon runAddonFor:[myTextView string] range:[myTextView selectedRange] type:&type path:[self fileName]];
		if(type == KReplaceSelection){
			[myTextView insertText:resString];
		}else if(type == KReplaceAll){
			[myTextView setString:resString];
		}else{
			// do nothing.
		}
		
	}
}

- (IBAction)showInformationDialog:(id)sender
{
	//APPKIT_EXTERN void NSBeginAlertSheet(
	//		NSString *title, NSString *defaultButton, NSString *alternateButton, NSString *otherButton,
	//		NSWindow *docWindow, id modalDelegate,
	//		SEL didEndSelector, SEL didDismissSelector,
	//		void *contextInfo, NSString *msg, ...);
	
	unsigned stringLength = [[myTextView string] length];
	unsigned lineCount = [KStringUtils countLinesOfString:[myTextView string]];
	NSRange selectedRange = [myTextView selectedRange];
	NSString* substring = [[myTextView string] substringWithRange:selectedRange];
	unsigned substringLength = [substring length];
	unsigned sublineCount = [KStringUtils countLinesOfString:substring];
	
	NSString* alertString = [NSString stringWithFormat:
					@"Whole Text :\n  Character Count : %d\n  Line Count : %d\n\n"
					@"Selected Text :\n  Character Count : %d\n  Line Count : %d",
					stringLength, lineCount,
					substringLength, sublineCount];

	NSBeginAlertSheet(
            @"Text Information", 
            @"OK", 
            nil, 
            nil, 
            myWindow, 
            self, 
			@selector(showInformationDialogEnded:returnCode:contextInfo:),
            nil, 
            nil, 
            alertString);
}

- (IBAction)showInformationDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	//Do nothing...
	//NSLog(@"showInformationDialogEnded:::");
}



- (IBAction)forceToChangeEncodingDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	if(returnCode == 1){ // pressed 'OK' button.
		NSData* data = [NSData dataWithContentsOfFile:[self fileName]];
		NSString* bufString = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
		NSString* modString = [KStringUtils convertReturnsToLF:bufString from:nil];
		[myTextView setString:modString];
		[myTextView setSelectedRange:NSMakeRange(0, 0)];
		[bufString release];
	}
}

//Actions for Search Functions.
- (IBAction)searchShowWindow:(id)sender
{
	id searchServer = [[NSApp delegate] searchServer];
	[searchServer showSearchWindow:self];
}
- (IBAction)searchForward:(id)sender
{
	[self doSearch:self direction:KSearchForward];
}
- (IBAction)searchBackward:(id)sender
{
	[self doSearch:self direction:KSearchBackward];
}
- (IBAction)setSelectionToSearch:(id)sender
{
	NSString* currentString = [[myTextView textStorage] string];
	NSRange selectedRange = [myTextView selectedRange];
	[[[NSApp delegate] searchServer] setSearchString:[currentString substringWithRange:selectedRange]];
	
}

- (IBAction)replace:(id)sender
{
	KSearchServer* searchServer = [[NSApp delegate] searchServer];
	NSRange selectedRange = [myTextView selectedRange];
	if(selectedRange.length != 0){
		[myTextView insertText:[searchServer replaceString]];
		[myTextView setSelectedRange:NSMakeRange(selectedRange.location, [[searchServer replaceString] length])];
	}
}

- (IBAction)replaceAndFindAgain:(id)sender
{
	KSearchServer* searchServer = [[NSApp delegate] searchServer];
	if([myTextView selectedRange].length != 0){
		[myTextView insertText:[searchServer replaceString]];
	}
	[self doSearch:self direction:KSearchForward];
}

//Actions for BottomView
- (IBAction)setSelectedRangeOfTextView:(id)sender
{
	NSRange range = [[sender representedObject] range];
	[myTextView setSelectedRange:range];
	[myTextView scrollRangeToVisible:range];
}

//Actions for GotoLineWindow
- (IBAction)setCurrentLineOfTextView:(id)sender
{
	if(![[sender stringValue] isEqualToString:@""]){
		[myTextView selectLineOf:[[sender stringValue] intValue]];
	}
	[[NSApp delegate] hideGotoLineWindow];
}


- (IBAction)showGotoLineWindow:(id)sender
{
	[[NSApp delegate] showGotoLineWindow];
}


@end



@implementation MyDocument (ColorSyntax)

- (void)validateColorSyntax
{
	[self validateColorSyntax:NSMakeRange(0, [[myTextView string] length])];
}

- (void)validateColorSyntax:(NSRange)validatedRange
{
	[mColorSyntaxServer adjustColorSyntaxWithRange:validatedRange andFont:[self textViewFont]];
}

@end


@implementation MyDocument (ScriptingSupport)

- (NSString*)retcode
{
	switch(mReturnCodeType){
		case KLF_type: return @"LF";
		case KCR_type: return @"CR";
		case KCRLF_type: return @"CR-LF";
	}
	[NSException raise:NSGenericException format:@"MyDocument - retcode"];
	return nil;
}

- (void)setRetcode:(NSString*)newRetcode
{
	if([newRetcode isEqualToString:@"LF"]){
		mReturnCodeType = KLF_type;
	}else if([newRetcode isEqualToString:@"CR"]){
		mReturnCodeType = KCR_type;
	}else if([newRetcode isEqualToString:@"CR-LF"]){
		mReturnCodeType = KCRLF_type;
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setRetcode"];
	}
}

- (NSString*)syntaxtype
{
	switch(mTextType){
		case KDocumentTextFileFormat_Plain: return KPlainName;
		case KDocumentTextFileFormat_HTML: return KHTMLName;
		case KDocumentTextFileFormat_Indexed: return KIndexName;
		case KDocumentTextFileFormat_KIndex: return KKEditMarkupName;
		case KDocumentTextFileFormat_CSS: return KCSSName;
		case KDocumentTextFileFormat_Perl: return KPerlScriptName;
		case KDocumentTextFileFormat_PHP: return KPHPName;
		case KDocumentTextFileFormat_Ruby: return KRubyScriptName;
		
		case KDocumentTextFileFormat_SH: return KShellScriptName;
	}
	[NSException raise:NSGenericException format:@"MyDocument - syntaxtype"];
	return nil;
}

- (void)setSyntaxtype:(NSString*)newSyntaxType
{
	if([newSyntaxType isEqualToString:KPlainName]){
		[self setTextType:KDocumentTextFileFormat_Plain];
	}else if([newSyntaxType isEqualToString:KHTMLName]){
		[self setTextType:KDocumentTextFileFormat_HTML];
	}else if([newSyntaxType isEqualToString:KIndexName]){
		[self setTextType:KDocumentTextFileFormat_Indexed];
	}else if([newSyntaxType isEqualToString:KKEditMarkupName]){
		[self setTextType:KDocumentTextFileFormat_KIndex];
	}else if([newSyntaxType isEqualToString:KCSSName]){
		[self setTextType:KDocumentTextFileFormat_CSS];
	}else if([newSyntaxType isEqualToString:KPerlScriptName]){
		[self setTextType:KDocumentTextFileFormat_Perl];
	}else if([newSyntaxType isEqualToString:KRubyScriptName]){
		[self setTextType:KDocumentTextFileFormat_Ruby];
	}else if([newSyntaxType isEqualToString:KPHPName]){
		[self setTextType:KDocumentTextFileFormat_PHP];
	
	}else if([newSyntaxType isEqualToString:KShellScriptName]){
    
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setSyntaxType"];
		return;
	}
	[self adjustDocumentSettings];
	[self validateColorSyntax];
}

- (NSString*)fontname
{
	return [NSString stringWithString:[self textViewFontName]];
}

- (void)setFontname:(NSString*)newFont
{
	// no error treatment.
	[self setTextViewFontName:newFont];
	[self validateColorSyntax];
}

- (NSNumber*)fontsize
{
	return [NSNumber numberWithFloat:[self textViewFontSize]];
}

- (void)setFontsize:(NSNumber*)newSize
{
	float size = [newSize floatValue];
	if(size >= 1.0){
		[self setTextViewFontSize:size];
		[self validateColorSyntax];
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setFontsize"];
	}
}

- (NSDictionary*)selectedrange
{
	NSRange range = [myTextView selectedRange];
	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithInt:range.location], @"uni_location",
				[NSNumber numberWithInt:range.length], @"uni_length", nil];
}

- (void)setSelectedrange:(NSDictionary*)newSelectedRange
{
	NSRange range;
	unsigned stringLength = [[myTextView string] length];
	NSNumber* location, *length;
	if((location = [newSelectedRange objectForKey:@"uni_location"]) && (length = [newSelectedRange objectForKey:@"uni_length"])){
		range.location = [location intValue];
		range.length = [length intValue];
		if(range.location >= 0 && range.length >= 0 && NSMaxRange(range) <= stringLength){
			[myTextView setSelectedRange:range];
		}else{
			[NSException raise:NSRangeException format:@"Given Range is not Valid."];
		}
			
	}
}

- (NSNumber*)wordwrap
{
	return [NSNumber numberWithBool:[self isWordWrapped]];
}

- (void)setWordwrap:(NSNumber*)newWordWrap
{
	[self setWordWrapped:[newWordWrap boolValue]];
}

- (NSNumber*)autoindent
{
	return [NSNumber numberWithBool:[self isAutoIndented]];
}

- (void)setAutoindent:(NSNumber*)newAutoIndent
{
	[self setAutoIndented:[newAutoIndent boolValue]];
}

- (NSTextStorage*)selectedtext
{
	//NSString* selectedText = [[myTextView string] substringWithRange:[myTextView selectedRange]];
	//return [NSString stringWithString:selectedText];
	NSAttributedString* attrString = [[myTextView textStorage] attributedSubstringFromRange:[myTextView selectedRange]];
	NSTextStorage * storage = [[NSTextStorage alloc] initWithAttributedString:attrString];
	return [storage autorelease];
}

- (void)setSelectedtext:(NSTextStorage*)newText
{
	if(newText){
		NSRange selectedRange = [myTextView selectedRange];
		[myTextView replaceCharactersInRange:selectedRange withString:[newText string]];
		[myTextView setSelectedRange:NSMakeRange(selectedRange.location, [newText length])];
	}
}

- (NSNumber*)selectionLocation
{
	return [NSNumber numberWithInt:[myTextView selectedRange].location];
}

- (void)setSelectionLocation:(NSNumber*)newSelectionLocation
{
	NSRange selectedRange = [myTextView selectedRange];
	int maxLength = [[myTextView string] length];
	int newLocation = [newSelectionLocation intValue];
	if(newLocation > maxLength){
		[NSException raise:NSGenericException format:@"MyDocument - setSelectionLocation"];
		return;
	}
	if(newLocation + selectedRange.length > maxLength){
		selectedRange.length = maxLength - newLocation;
	}
	selectedRange.location = newLocation;
	[myTextView setSelectedRange:selectedRange];
}

- (NSNumber*)selectionLength
{
	return [NSNumber numberWithInt:[myTextView selectedRange].length];
}

- (void)setSelectionLength:(NSNumber*)newSelectionLength
{
	NSRange selectedRange = [myTextView selectedRange];
	selectedRange.length = [newSelectionLength intValue];
	int maxLength = [[myTextView string] length];

	if(selectedRange.length > maxLength - selectedRange.location){
		[NSException raise:NSGenericException format:@"MyDocument - setSelectionLength"];
		return;
	}
	[myTextView setSelectedRange:selectedRange];
}

- (NSTextStorage*)text
{
	return [myTextView textStorage];
}




- (NSNumber*)handleReplaceAllCommand:(NSScriptCommand*)command
{
	NSDictionary* dict = [command evaluatedArguments];
	
	BOOL ignoreCase = YES;
	NSString* searchText = nil;
	NSString* replaceText = nil;
	
	if(!(searchText = [dict objectForKey:@"searchText"]) 
			|| !(replaceText = [dict objectForKey:@"replaceText"])
			|| [searchText isEqualToString:@""]){
		[NSException raise:NSGenericException format:@"MyDocument - handleReplaceAllCommand:"];
	}
	
	id curObj;
	if(curObj = [dict objectForKey:@"ignoreCase"]){
		ignoreCase = [curObj boolValue];
	}
	KSearchServer* searchServer = [[KSearchServer alloc] init];
	[searchServer setSearchString:searchText];
	[searchServer setReplaceString:replaceText];
	[searchServer setIgnoreCase:ignoreCase];
	
	unsigned long replaceCount = [searchServer replaceAll:myTextView];
	
	[searchServer release];
	
	return [NSNumber numberWithLong:replaceCount];
}

- (NSNumber*)handleSearchCommand:(NSScriptCommand*)command
{
	NSDictionary* dict = [command evaluatedArguments];
	
	BOOL ignoreCase = YES;
	NSString* searchText = nil;
	unsigned long fromOffset = 0;
	id curObj;
	if(!(searchText = [dict objectForKey:@"searchText"])
			|| [searchText isEqualToString:@""]){
		[NSException raise:NSGenericException format:@"MyDocument - handleSearchCommand:"];
	}
	if(curObj = [dict objectForKey:@"ignoreCase"]){
		ignoreCase = [curObj boolValue];
	}
	if(curObj = [dict objectForKey:@"fromOffset"]){
		fromOffset = [curObj longValue];
		if(fromOffset > [[myTextView string] length]){
			[NSException raise:NSGenericException format:@"MyDocument - illegal 'fromOffset'"];
		}
	}
	
	KSearchServer* searchServer = [[KSearchServer alloc] init];
	[searchServer setSearchString:searchText];
	[searchServer setIgnoreCase:ignoreCase];
	
	NSRange currentSelectedRange = [myTextView selectedRange];
	[myTextView setSelectedRange:NSMakeRange(fromOffset, 0)];
	NSRange newSelection = [searchServer searchString:myTextView direction:KSearchForward];
	[myTextView setSelectedRange:currentSelectedRange];
	
	[searchServer release];
	
	if(newSelection.location == NSNotFound){
		return [NSNumber numberWithLong:-1];
	}
	
	return [NSNumber numberWithLong:newSelection.location];
}

- (void)handleScrollToSelectionCommand:(NSScriptCommand*)command
{
	[self scrollToSelection:nil];
}


@end

















